if (typeof browser !== 'undefined') {
    chrome = browser;
}

var rc2CanCloseIntervalHandles = Object.create(null);

chrome.tabs.onRemoved.addListener(function (tab) {
    if (rc2CanCloseIntervalHandles[tab.id] != null) {
        removeRc2CanCloseCheck(tab.id);
    }
});

chrome.runtime.onMessage.addListener(
    function (request, sender, sendResponse) {
        if (request.name === "myjdrc2") {
            if (request.action === "tabmode-init") {
                onTabmodeInitEvent(request, sender);
            } else if (request.action === "response") {
                onResponseEvent(request, sender);
            } else if (request.action === "unload") {
                if (request.data.callbackUrl != null) {
                    onRc2TabUnload(request, sender);
                }
            } else if (request.action === "loaded") {
                if (request.callbackUrl != null) {
                    onRc2FrameLoaded(request, sender);
                }
            } else if (request.action === "mouse-move") {
                onRc2MouseMove(request.callbackUrl, request.timestamp);
            }
        } else if (request.name === "close-me") {
            onCloseMeEvent(request, sender);
        }
        return true;
    });

var onCloseMeEvent = function (request, sender) {
    if (request.data && request.data.tabId) {
        var tabId = Number.parseInt(request.data.tabId);
        if (request.data.tabId === "self") {
            tabId = sender.tab.id;
        }
        chrome.tabs.get(tabId, function (tab) {
            chrome.tabs.remove(tabId);
            chrome.windows.get(tab.windowId, {populate: true}, function (window) {
                var isNewTab = false;
                if (window.tabs.length === 1) {
                    isNewTab = window.tabs[0].url === "about:newtab";
                }
                if (window.tabs.length === 0 || isNewTab) {
                    chrome.windows.remove(window.id);
                }
            });
        });
    }
};

var onRc2TabUnload = function (request, sender) {
    var xhr = new XMLHttpRequest();
    xhr.open("GET", request.data.callbackUrl + "&do=unload", true);
    xhr.timeout = 5000;
    xhr.send();
};

var onRc2FrameLoaded = function (request, sender) {
    var params = request.params;
    var callbackUrl = request.callbackUrl;
    var xhr = new XMLHttpRequest();
    var urlParams = "&x=" + params.x + "&y=" + params.y + "&w=" + params.w + "&h=" + params.h + "&vw=" + params.vw + "&vh=" + params.vh + "&eleft=" + params.eleft + "&etop=" + params.etop + "&ew=" + params.ew + "&eh=" + params.eh;
    xhr.open("GET", callbackUrl + "&do=loaded" + urlParams, true);
    xhr.timeout = 5000;
    xhr.send();
};

var onResponseEvent = function (request, sender) {
    if (request.data.callbackUrl !== undefined && request.data.callbackUrl !== "undefined") {
        // send the solution to local JDownloader
        var xhr = new XMLHttpRequest();
        xhr.ontimeout = function () {
            console.error("The request for " + request.data.callbackUrl + " timed out.");
        };
        xhr.onload = function () {
            if (sender && sender.tab && sender.tab.id) {
                setTimeout(function () {
                    chrome.tabs.remove(sender.tab.id);
                }, 2000);
            }
        };
        xhr.open("GET", request.data.callbackUrl + "&do=solve&response=" + request.data.token, true);
        xhr.send();
    }
};

var checkCanClose = function (request, sender) {
    var xhr = new XMLHttpRequest();
    xhr.ontimeout = function () {
        console.error("The request for " + request.data.callbackUrl + " timed out.");
        removeRc2CanCloseCheck(sender.tab.id);
    };
    xhr.onreadystatechange = function () {
        if (this.readyState == 4) {
            if (this.status == 200) {
                if (xhr.response === "true") {
                    removeRc2CanCloseCheck(sender.tab.id);
                    chrome.tabs.remove(sender.tab.id);
                }
            } else {
                // Exceptional response from JD
                removeRc2CanCloseCheck(sender.tab.id);
                chrome.tabs.remove(sender.tab.id);
            }
        }
    };
    xhr.open("GET", request.data.callbackUrl + "&do=canClose", true);
    xhr.send();
};

function onRc2MouseMove(callbackUrl, timestamp) {
    var xhr = new XMLHttpRequest();
    xhr.open("GET", callbackUrl + "&do=canClose&useractive=true&ts=" + timestamp, true);
    xhr.timeout = 5000;
    xhr.send();
}

var removeRc2CanCloseCheck = function (tabId) {
    if (rc2CanCloseIntervalHandles[tabId] !== undefined) {
        clearInterval(rc2CanCloseIntervalHandles[tabId]);
        delete rc2CanCloseIntervalHandles[tabId];
    }
};

var onTabmodeInitEvent = function (request, sender) {
    if (request.data.callbackUrl != null) {
        var handle = setInterval(function () {
            if (!rc2CanCloseIntervalHandles[sender.tab.id]) {
                clearInterval(handle);
            }
            checkCanClose(request, sender);
        }, 1000);
        rc2CanCloseIntervalHandles[sender.tab.id] = handle;
    }
};